#ifndef _RASPELL_GLOBAL_H
#define _RASPELL_GLOBAL_H

#include <ruby.h>
#include <aspell.h>

VALUE cAspell;
VALUE cDictInfo;

VALUE cAspellError;

#endif

/**
 * raspell      September 2002      
 *
 * Copyright (c) aquanauten (resp: matthias_veit@yahoo.de)
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option) 
 * any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 */
