
require "raspell"

aspell = Aspell.new1("lang"=>"en", "size"=>"50") 

text = ["Hiere is somthing wrong on the planett. And it was not the Apollo."]

p aspell.correct_lines(text) { |word| "<wrong word>" }
